from .DydxPerpetualMain import DydxPerpetualMain
from jesse.enums import exchanges


class DydxPerpetual(DydxPerpetualMain):
    def __init__(self) -> None:
        from jesse.modes.import_candles_mode.drivers.Bitfinex.BitfinexSpot import BitfinexSpot
        super().__init__(
            name=exchanges.DYDX_PERPETUAL,
            rest_endpoint='https://api.dydx.exchange',
            backup_exchange_class=BitfinexSpot
        )



from typing import Union
import requests
from jesse.modes.import_candles_mode.drivers.interface import CandleExchange
from .bitget_spot_utils import timeframe_to_interval
import jesse.helpers as jh
from jesse.enums import exchanges
from jesse import exceptions


class BitgetSpot(CandleExchange):
    def __init__(self) -> None:
        super().__init__(
            name=exchanges.BITGET_SPOT,
            count=100,
            rate_limit_per_second=18,
            backup_exchange_class=None
        )

        self.endpoint = 'https://api.bitget.com/api/spot/v1/market/candles'

    def get_starting_time(self, symbol: str) -> int:
        payload = {
            'after': 1359291660000,
            'before': jh.now(force_fresh=True),
            'period': '1week',
            'symbol': self._jesse_symbol_to_bitget_usdt_contracts_symbol(symbol),
        }

        response = requests.get(self.endpoint, params=payload)

        self.validate_bitget_response(response)

        data = response.json()

        # since the first timestamp doesn't include all the 1m
        # candles, let's start since the second day then
        return int(data[1][0])

    def fetch(self, symbol: str, start_timestamp: int, timeframe: str = '1m') -> Union[list, None]:
        end_timestamp = start_timestamp + (self.count - 1) * 60000 * jh.timeframe_to_one_minutes(timeframe)

        payload = {
            'period': timeframe_to_interval(timeframe),
            'symbol': self._jesse_symbol_to_bitget_usdt_contracts_symbol(symbol),
            'after': int(start_timestamp),
            'before': int(end_timestamp)
        }

        response = requests.get(self.endpoint, params=payload)

        self.validate_bitget_response(response)

        data = response.json()

        return [
            {
                'id': jh.generate_unique_id(),
                'exchange': self.name,
                'symbol': symbol,
                'timeframe': timeframe,
                'timestamp': int(d['ts']),
                'open': float(d['open']),
                'high': float(d['high']),
                'low': float(d['low']),
                'close': float(d['close']),
                'volume': float(d['baseVol'])
            } for d in data
        ]

    @staticmethod
    def _jesse_symbol_to_bitget_usdt_contracts_symbol(symbol: str) -> str:
        return f'{jh.dashless_symbol(symbol)}_SPBL'

    def validate_bitget_response(self, response):
        data = response.json()

        # 40019: wrong symbol
        if response.status_code == 400 and data['code'] == "40019":
            msg = 'Symbol not found. Check the symbol and try again.'
            msg += f' Example of a valid symbol for "{self.name}": "BTC-USDT"'
            raise exceptions.SymbolNotFound(msg)

        self.validate_response(response)



from starlette.responses import JSONResponse

from jesse.modes.import_candles_mode import CandleExchange
from jesse.modes.import_candles_mode.drivers import drivers, driver_names


def get_exchange_supported_symbols(exchange: str) -> JSONResponse:
    arr = []

    try:
        driver: CandleExchange = drivers[exchange]()
    except KeyError:
        raise ValueError(f'{exchange} is not a supported exchange. Supported exchanges are: {driver_names}')

    try:
        arr = driver.get_available_symbols()
    except Exception as e:
        return JSONResponse({
            'error': str(e)
        }, status_code=500)

    return JSONResponse({
        'data': arr
    }, status_code=200)



from jesse.enums import timeframes


def timeframe_to_interval(timeframe: str) -> str:
    # 1m
    # 3m
    # 5m
    # 15m
    # 30m
    # 1h
    # 2h
    # 4h
    # 6h
    # 8h
    # 12h
    # 1d
    # 3d
    # 1w
    # 1M
    if timeframe == timeframes.MINUTE_1:
        return '1m'
    elif timeframe == timeframes.MINUTE_3:
        return '3m'
    elif timeframe == timeframes.MINUTE_5:
        return '5m'
    elif timeframe == timeframes.MINUTE_15:
        return '15m'
    elif timeframe == timeframes.MINUTE_30:
        return '30m'
    elif timeframe == timeframes.HOUR_1:
        return '1h'
    elif timeframe == timeframes.HOUR_2:
        return '2h'
    elif timeframe == timeframes.HOUR_4:
        return '4h'
    elif timeframe == timeframes.HOUR_6:
        return '6h'
    elif timeframe == timeframes.HOUR_8:
        return '8h'
    elif timeframe == timeframes.HOUR_12:
        return '12h'
    elif timeframe == timeframes.DAY_1:
        return '1d'
    elif timeframe == timeframes.DAY_3:
        return '3d'
    elif timeframe == timeframes.WEEK_1:
        return '1w'
    elif timeframe == timeframes.MONTH_1:
        return '1M'
    else:
        raise ValueError('Invalid timeframe: {}'.format(timeframe))


def interval_to_timeframe(interval: str) -> str:
    if interval == '1m':
        return timeframes.MINUTE_1
    elif interval == '3m':
        return timeframes.MINUTE_3
    elif interval == '5m':
        return timeframes.MINUTE_5
    elif interval == '15m':
        return timeframes.MINUTE_15
    elif interval == '30m':
        return timeframes.MINUTE_30
    elif interval == '1h':
        return timeframes.HOUR_1
    elif interval == '2h':
        return timeframes.HOUR_2
    elif interval == '4h':
        return timeframes.HOUR_4
    elif interval == '6h':
        return timeframes.HOUR_6
    elif interval == '8h':
        return timeframes.HOUR_8
    elif interval == '12h':
        return timeframes.HOUR_12
    elif interval == '1d':
        return timeframes.DAY_1
    elif interval == '3d':
        return timeframes.DAY_3
    elif interval == '1w':
        return timeframes.WEEK_1
    elif interval == '1M':
        return timeframes.MONTH_1
    else:
        raise ValueError('Invalid interval: {}'.format(interval))
